// Generated by CoffeeScript 1.10.0
(function() {
  var Server, chokidar, defaultExclusions, defaultExts, defaultPort, fs, http, https, path, protocol_version, url, ws;

  fs = require('fs');

  path = require('path');

  ws = require('ws');

  http = require('http');

  https = require('https');

  url = require('url');

  chokidar = require('chokidar');

  protocol_version = '7';

  defaultPort = 35729;

  defaultExts = ['html', 'css', 'js', 'png', 'gif', 'jpg', 'php', 'php5', 'py', 'rb', 'erb', 'coffee'];

  defaultExclusions = [/\\.git\//, /\\.svn\//, /\\.hg\//];

  Server = (function() {
    function Server(config1) {
      var base, base1, base2, base3, base4, base5, base6, base7;
      this.config = config1;
      if (this.config == null) {
        this.config = {};
      }
      if ((base = this.config).version == null) {
        base.version = protocol_version;
      }
      if ((base1 = this.config).port == null) {
        base1.port = defaultPort;
      }
      if ((base2 = this.config).exts == null) {
        base2.exts = [];
      }
      if ((base3 = this.config).exclusions == null) {
        base3.exclusions = [];
      }
      this.config.exts = this.config.exts.concat(defaultExts);
      this.config.exclusions = this.config.exclusions.concat(defaultExclusions);
      if ((base4 = this.config).applyCSSLive == null) {
        base4.applyCSSLive = true;
      }
      if ((base5 = this.config).originalPath == null) {
        base5.originalPath = '';
      }
      if ((base6 = this.config).overrideURL == null) {
        base6.overrideURL = '';
      }
      if ((base7 = this.config).usePolling == null) {
        base7.usePolling = false;
      }
    }

    Server.prototype.listen = function(callback) {
      this.debug("LiveReload is waiting for browser to connect.");
      if (this.config.server) {
        this.config.server.listen(this.config.port);
        this.server = new ws.Server({
          server: this.config.server
        });
      } else {
        this.server = new ws.Server({
          port: this.config.port
        });
      }
      this.server.on('connection', this.onConnection.bind(this));
      this.server.on('close', this.onClose.bind(this));
      if (callback) {
        return this.server.once('listening', callback);
      }
    };

    Server.prototype.onConnection = function(socket) {
      this.debug("Browser connected.");
      socket.on('message', (function(_this) {
        return function(message) {
          var data, request;
          _this.debug("Client message: " + message);
          request = JSON.parse(message);
          if (request.command === "hello") {
            _this.debug("Client requested handshake...");
            _this.debug("Handshaking with client using protocol " + _this.config.version + "...");
            data = JSON.stringify({
              command: 'hello',
              protocols: ['http://livereload.com/protocols/official-7', 'http://livereload.com/protocols/official-8', 'http://livereload.com/protocols/official-9', 'http://livereload.com/protocols/2.x-origin-version-negotiation', 'http://livereload.com/protocols/2.x-remote-control'],
              serverName: 'node-livereload'
            });
            return socket.send(data);
          }
        };
      })(this));
      socket.on('error', (function(_this) {
        return function(err) {
          return _this.debug("Error in client socket: " + err);
        };
      })(this));
      return socket.on('close', (function(_this) {
        return function(message) {
          return _this.debug("Client closed connection");
        };
      })(this));
    };

    Server.prototype.onClose = function(socket) {
      return this.debug("Socket closed.");
    };

    Server.prototype.watch = function(paths) {
      return this.watcher = chokidar.watch(paths, {
        ignoreInitial: true,
        ignored: this.config.exclusions,
        usePolling: this.config.usePolling
      }).on('add', this.filterRefresh.bind(this)).on('change', this.filterRefresh.bind(this)).on('unlink', this.filterRefresh.bind(this));
    };

    Server.prototype.filterRefresh = function(filepath) {
      var delayedRefresh, exts, fileext;
      exts = this.config.exts;
      fileext = path.extname(filepath).substring(1);
      if (exts.indexOf(fileext) !== -1) {
        if (this.config.delay) {
          return delayedRefresh = setTimeout((function(_this) {
            return function() {
              clearTimeout(delayedRefresh);
              return _this.refresh(filepath);
            };
          })(this), this.config.delay);
        } else {
          return this.refresh(filepath);
        }
      }
    };

    Server.prototype.refresh = function(filepath) {
      var data, i, len, ref, results, socket;
      this.debug("Refresh: " + filepath);
      data = JSON.stringify({
        command: 'reload',
        path: filepath,
        liveCSS: this.config.applyCSSLive,
        liveImg: this.config.applyImgLive,
        originalPath: this.config.originalPath,
        overrideURL: this.config.overrideURL
      });
      ref = this.server.clients;
      results = [];
      for (i = 0, len = ref.length; i < len; i++) {
        socket = ref[i];
        results.push(socket.send(data, (function(_this) {
          return function(error) {
            if (error) {
              return _this.debug(error);
            }
          };
        })(this)));
      }
      return results;
    };

    Server.prototype.debug = function(str) {
      if (this.config.debug) {
        return console.log(str + "\n");
      }
    };

    Server.prototype.close = function() {
      this.watcher.close();
      this.server._server.close();
      return this.server.close();
    };

    return Server;

  })();

  exports.createServer = function(config, callback) {
    var app, requestHandler, server;
    if (config == null) {
      config = {};
    }
    requestHandler = function(req, res) {
      if (url.parse(req.url).pathname === '/livereload.js') {
        res.writeHead(200, {
          'Content-Type': 'text/javascript'
        });
        return res.end(fs.readFileSync(__dirname + '/../ext/livereload.js'));
      }
    };
    if (config.https == null) {
      app = http.createServer(requestHandler);
    } else {
      app = https.createServer(config.https, requestHandler);
    }
    if (config.server == null) {
      config.server = app;
    }
    server = new Server(config);
    if (!config.noListen) {
      server.listen(callback);
    }
    return server;
  };

}).call(this);
